@@ -1,19 +1,19 @@
 use std::str::FromStr;
 
-use crate::commands::KeyCommand;
+use crate::config::AppKeyMapping;
 use crate::context::AppContext;
 use crate::error::JoshutoResult;
+use crate::key_command::{AppExecute, Command};
 use crate::ui::views::TuiTextField;
-use crate::ui::TuiBackend;
+use crate::ui::AppBackend;
 
-use super::AppExecute;
-
-pub fn readline(
+pub fn read_and_execute(
     context: &mut AppContext,
-    backend: &mut TuiBackend,
+    backend: &mut AppBackend,
+    keymap_t: &AppKeyMapping,
     prefix: &str,
     suffix: &str,
-) -> JoshutoResult<()> {
+) -> JoshutoResult {
     context.flush_event();
     let user_input: Option<String> = TuiTextField::default()
         .prompt(":")
@@ -23,8 +23,9 @@ pub fn readline(
 
     if let Some(s) = user_input {
         let trimmed = s.trim_start();
-        let command = KeyCommand::from_str(trimmed)?;
-        command.execute(context, backend)
+        context.commandline_context_mut().history_mut().add(trimmed);
+        let command = Command::from_str(trimmed)?;
+        command.execute(context, backend, keymap_t)
     } else {
         Ok(())
     }
