@@ -6,23 +6,18 @@ use termion::event::Key;
 use crate::context::AppContext;
 use crate::history::DirectoryHistory;
 use crate::ui::widgets::TuiPrompt;
-use crate::ui::TuiBackend;
+use crate::ui::AppBackend;
 
 use super::reload;
 
 fn trash_error_to_io_error(err: trash::Error) -> std::io::Error {
     match err {
-        trash::Error::Unknown => std::io::Error::new(std::io::ErrorKind::Other, "Unknown Error"),
+        trash::Error::Unknown { description } => {
+            std::io::Error::new(std::io::ErrorKind::Other, description)
+        }
         trash::Error::TargetedRoot => {
             std::io::Error::new(std::io::ErrorKind::Other, "Targeted Root")
         }
-        trash::Error::CanonicalizePath { code: _ } => {
-            std::io::Error::new(std::io::ErrorKind::NotFound, "Not found")
-        }
-        trash::Error::Remove { code: Some(1) } => std::io::Error::new(
-            std::io::ErrorKind::InvalidData,
-            "Cannot move files to trash from mounted system",
-        ),
         _ => std::io::Error::new(std::io::ErrorKind::Other, "Unknown Error"),
     }
 }
@@ -55,7 +50,7 @@ where
     Ok(())
 }
 
-fn delete_files(context: &mut AppContext, backend: &mut TuiBackend) -> std::io::Result<()> {
+fn delete_files(context: &mut AppContext, backend: &mut AppBackend) -> std::io::Result<()> {
     let delete_func = if context.config_ref().use_trash {
         trash_files
     } else {
@@ -68,7 +63,7 @@ fn delete_files(context: &mut AppContext, backend: &mut TuiBackend) -> std::io::
         .curr_tab_ref()
         .curr_list_ref()
         .map(|s| s.get_selected_paths())
-        .unwrap_or_else(|| vec![]);
+        .unwrap_or_else(Vec::new);
     let paths_len = paths.len();
     if paths_len == 0 {
         return Err(std::io::Error::new(
@@ -84,7 +79,7 @@ fn delete_files(context: &mut AppContext, backend: &mut TuiBackend) -> std::io::
     };
 
     match ch {
-        Key::Char('y') | Key::Char('\n') => {
+        Key::Char('Y') | Key::Char('y') | Key::Char('\n') => {
             let confirm_delete = if paths_len > 1 {
                 // prompt user again for deleting multiple files
                 let ch = {
@@ -117,7 +112,7 @@ fn delete_files(context: &mut AppContext, backend: &mut TuiBackend) -> std::io::
 
 pub fn delete_selected_files(
     context: &mut AppContext,
-    backend: &mut TuiBackend,
+    backend: &mut AppBackend,
 ) -> std::io::Result<()> {
     let _ = delete_files(context, backend)?;
 
