@@ -4,28 +4,28 @@ pub mod command_line;
 pub mod cursor_move;
 pub mod delete_files;
 pub mod file_ops;
+pub mod flat;
+pub mod line_nums;
 pub mod new_directory;
+pub mod numbered_command;
 pub mod open_file;
 pub mod parent_cursor_move;
-pub mod parent_directory;
+pub mod preview_cursor_move;
 pub mod quit;
 pub mod reload;
 pub mod rename_file;
 pub mod search;
+pub mod search_fzf;
 pub mod search_glob;
-pub mod search_skim;
 pub mod search_string;
 pub mod selection;
 pub mod set_mode;
+pub mod show_help;
 pub mod show_hidden;
-pub mod show_workers;
+pub mod show_tasks;
 pub mod sort;
 pub mod sub_process;
+pub mod subdir_fzf;
 pub mod tab_ops;
 pub mod touch_file;
-
-pub mod command_keybind;
-pub mod key_command;
-
-pub use self::command_keybind::{AppCommand, AppExecute, CommandKeybind};
-pub use self::key_command::KeyCommand;
+pub mod zoxide;
