@@ -4,7 +4,7 @@ use crate::context::AppContext;
 use crate::error::JoshutoResult;
 use crate::history::DirectoryHistory;
 
-pub fn new_directory(context: &mut AppContext, p: &path::Path) -> JoshutoResult<()> {
+pub fn new_directory(context: &mut AppContext, p: &path::Path) -> JoshutoResult {
     std::fs::create_dir_all(p)?;
     let options = context.config_ref().display_options_ref().clone();
     let curr_path = context.tab_context_ref().curr_tab_ref().cwd().to_path_buf();
