@@ -1,8 +1,10 @@
 use std::path;
 
+use crate::config::AppKeyMapping;
 use crate::context::AppContext;
 use crate::error::JoshutoResult;
-use crate::ui::TuiBackend;
+use crate::history::create_dirlist_with_history;
+use crate::ui::AppBackend;
 
 use super::command_line;
 
@@ -17,14 +19,24 @@ pub fn _rename_file(
         return Err(err);
     }
     std::fs::rename(&src, &dest)?;
-    let options = context.config_ref().display_options_ref().clone();
-    if let Some(curr_list) = context.tab_context_mut().curr_tab_mut().curr_list_mut() {
-        curr_list.reload_contents(&options)?;
+
+    let path = context
+        .tab_context_ref()
+        .curr_tab_ref()
+        .curr_list_ref()
+        .map(|lst| lst.file_path().to_path_buf());
+
+    if let Some(path) = path {
+        let options = context.config_ref().display_options_ref().clone();
+
+        let history = context.tab_context_mut().curr_tab_mut().history_mut();
+        let new_dirlist = create_dirlist_with_history(history, path.as_path(), &options)?;
+        history.insert(path, new_dirlist);
     }
     Ok(())
 }
 
-pub fn rename_file(context: &mut AppContext, dest: &path::Path) -> JoshutoResult<()> {
+pub fn rename_file(context: &mut AppContext, dest: &path::Path) -> JoshutoResult {
     let path: Option<path::PathBuf> = context
         .tab_context_ref()
         .curr_tab_ref()
@@ -40,9 +52,10 @@ pub fn rename_file(context: &mut AppContext, dest: &path::Path) -> JoshutoResult
 
 pub fn _rename_file_append(
     context: &mut AppContext,
-    backend: &mut TuiBackend,
+    backend: &mut AppBackend,
+    keymap_t: &AppKeyMapping,
     file_name: &str,
-) -> JoshutoResult<()> {
+) -> JoshutoResult {
     let (prefix, suffix): (String, String) = match file_name.rfind('.') {
         Some(ext) => (
             format!("rename {}", &file_name[0..ext]),
@@ -50,10 +63,14 @@ pub fn _rename_file_append(
         ),
         None => (format!("rename {}", file_name), "".to_string()),
     };
-    command_line::readline(context, backend, &prefix, &suffix)
+    command_line::read_and_execute(context, backend, keymap_t, &prefix, &suffix)
 }
 
-pub fn rename_file_append(context: &mut AppContext, backend: &mut TuiBackend) -> JoshutoResult<()> {
+pub fn rename_file_append(
+    context: &mut AppContext,
+    backend: &mut AppBackend,
+    keymap_t: &AppKeyMapping,
+) -> JoshutoResult {
     let mut file_name: Option<String> = None;
 
     if let Some(curr_list) = context.tab_context_ref().curr_tab_ref().curr_list_ref() {
@@ -63,35 +80,35 @@ pub fn rename_file_append(context: &mut AppContext, backend: &mut TuiBackend) ->
     }
 
     if let Some(file_name) = file_name {
-        _rename_file_append(context, backend, file_name.as_str())?;
+        _rename_file_append(context, backend, keymap_t, file_name.as_str())?;
     }
     Ok(())
 }
 
 pub fn _rename_file_prepend(
     context: &mut AppContext,
-    backend: &mut TuiBackend,
+    backend: &mut AppBackend,
+    keymap_t: &AppKeyMapping,
     file_name: String,
-) -> JoshutoResult<()> {
+) -> JoshutoResult {
     let prefix = String::from("rename ");
     let suffix = file_name;
-    command_line::readline(context, backend, &prefix, &suffix)
+    command_line::read_and_execute(context, backend, keymap_t, &prefix, &suffix)
 }
 
 pub fn rename_file_prepend(
     context: &mut AppContext,
-    backend: &mut TuiBackend,
-) -> JoshutoResult<()> {
-    let mut file_name: Option<String> = None;
-
-    if let Some(curr_list) = context.tab_context_ref().curr_tab_ref().curr_list_ref() {
-        file_name = curr_list
-            .curr_entry_ref()
-            .map(|s| s.file_name().to_string());
-    }
+    backend: &mut AppBackend,
+    keymap_t: &AppKeyMapping,
+) -> JoshutoResult {
+    let file_name = context
+        .tab_context_ref()
+        .curr_tab_ref()
+        .curr_list_ref()
+        .and_then(|list| list.curr_entry_ref().map(|s| s.file_name().to_string()));
 
     if let Some(file_name) = file_name {
-        _rename_file_prepend(context, backend, file_name)?;
+        _rename_file_prepend(context, backend, keymap_t, file_name)?;
     }
     Ok(())
 }
