@@ -7,7 +7,7 @@ use crate::tab::{JoshutoTab, TabHomePage};
 
 use crate::HOME_DIR;
 
-use super::quit;
+use super::quit::{quit_with_action, QuitAction};
 
 fn _tab_switch(new_index: usize, context: &mut AppContext) -> std::io::Result<()> {
     context.tab_context_mut().index = new_index;
@@ -40,6 +40,12 @@ fn _tab_switch(new_index: usize, context: &mut AppContext) -> std::io::Result<()
         history.remove(cwd.as_path());
     }
 
+    if let Some(cwd_parent) = cwd.parent() {
+        if history.create_or_soft_update(cwd_parent, &options).is_err() {
+            history.remove(cwd_parent);
+        }
+    }
+
     if let Some(file_path) = entry_path {
         if history
             .create_or_soft_update(file_path.as_path(), &options)
@@ -60,7 +66,7 @@ pub fn tab_switch(offset: i32, context: &mut AppContext) -> std::io::Result<()>
     _tab_switch(new_index, context)
 }
 
-pub fn tab_switch_index(new_index: usize, context: &mut AppContext) -> JoshutoResult<()> {
+pub fn tab_switch_index(new_index: usize, context: &mut AppContext) -> JoshutoResult {
     if new_index <= context.tab_context_ref().len() {
         _tab_switch(new_index - 1, context)?;
         Ok(())
@@ -87,10 +93,14 @@ pub fn new_tab_home_path(context: &AppContext) -> path::PathBuf {
     }
 }
 
-pub fn new_tab(context: &mut AppContext) -> JoshutoResult<()> {
+pub fn new_tab(context: &mut AppContext) -> JoshutoResult {
     let new_tab_path = new_tab_home_path(context);
 
-    let tab = JoshutoTab::new(new_tab_path, context.config_ref().display_options_ref())?;
+    let tab = JoshutoTab::new(
+        new_tab_path,
+        context.ui_context_ref(),
+        context.config_ref().display_options_ref(),
+    )?;
     context.tab_context_mut().push_tab(tab);
     let new_index = context.tab_context_ref().len() - 1;
     context.tab_context_mut().index = new_index;
@@ -98,9 +108,9 @@ pub fn new_tab(context: &mut AppContext) -> JoshutoResult<()> {
     Ok(())
 }
 
-pub fn close_tab(context: &mut AppContext) -> JoshutoResult<()> {
+pub fn close_tab(context: &mut AppContext) -> JoshutoResult {
     if context.tab_context_ref().len() <= 1 {
-        return quit::quit(context);
+        return quit_with_action(context, QuitAction::Noop);
     }
     let mut tab_index = context.tab_context_ref().index;
 
