@@ -1,15 +1,26 @@
+use super::app_raw::AppConfigRaw;
+use super::DEFAULT_CONFIG_FILE_PATH;
+
 use crate::config::option::{DisplayOption, PreviewOption, SortOption, TabOption};
+use crate::error::JoshutoResult;
 
 #[derive(Debug, Clone)]
 pub struct AppConfig {
     pub use_trash: bool,
     pub xdg_open: bool,
+    pub xdg_open_fork: bool,
+    pub watch_files: bool,
     pub _display_options: DisplayOption,
     pub _preview_options: PreviewOption,
     pub _tab_options: TabOption,
 }
 
 impl AppConfig {
+    pub fn default_res() -> JoshutoResult<Self> {
+        let raw: AppConfigRaw = toml::from_str(DEFAULT_CONFIG_FILE_PATH)?;
+        Ok(Self::from(raw))
+    }
+
     pub fn display_options_ref(&self) -> &DisplayOption {
         &self._display_options
     }
@@ -20,7 +31,7 @@ impl AppConfig {
     pub fn preview_options_ref(&self) -> &PreviewOption {
         &self._preview_options
     }
-    pub fn preview_options_mut(&mut self) -> &mut PreviewOption {
+    pub fn _preview_options_mut(&mut self) -> &mut PreviewOption {
         &mut self._preview_options
     }
 
@@ -38,12 +49,8 @@ impl AppConfig {
 
 impl std::default::Default for AppConfig {
     fn default() -> Self {
-        Self {
-            use_trash: true,
-            xdg_open: false,
-            _display_options: DisplayOption::default(),
-            _preview_options: PreviewOption::default(),
-            _tab_options: TabOption::default(),
-        }
+        // This should not fail.
+        // If it fails then there is a (syntax) error in the default config file
+        Self::default_res().unwrap()
     }
 }
