@@ -1,4 +1,5 @@
 mod app_context;
+mod commandline_context;
 mod local_state;
 mod message_queue;
 mod preview_context;
@@ -6,6 +7,7 @@ mod tab_context;
 mod worker_context;
 
 pub use self::app_context::*;
+pub use self::commandline_context::*;
 pub use self::local_state::*;
 pub use self::message_queue::*;
 pub use self::preview_context::*;
