@@ -1,8 +1,11 @@
+use std::cmp::{min, Ordering};
+
 use tui::buffer::Buffer;
 use tui::layout::Rect;
 use tui::style::{Color, Modifier, Style};
 use tui::widgets::Widget;
 
+use crate::config::option::{DisplayOption, LineNumberStyle};
 use crate::fs::{FileType, JoshutoDirEntry, JoshutoDirList, LinkType};
 use crate::util::format;
 use crate::util::string::UnicodeTruncate;
@@ -15,11 +18,20 @@ const ELLIPSIS: &str = "…";
 
 pub struct TuiDirListDetailed<'a> {
     dirlist: &'a JoshutoDirList,
+    display_options: &'a DisplayOption,
+    pub focused: bool,
 }
-
 impl<'a> TuiDirListDetailed<'a> {
-    pub fn new(dirlist: &'a JoshutoDirList) -> Self {
-        Self { dirlist }
+    pub fn new(
+        dirlist: &'a JoshutoDirList,
+        display_options: &'a DisplayOption,
+        focused: bool,
+    ) -> Self {
+        Self {
+            dirlist,
+            display_options,
+            focused,
+        }
     }
 }
 
@@ -31,7 +43,7 @@ impl<'a> Widget for TuiDirListDetailed<'a> {
 
         let x = area.left();
         let y = area.top();
-        let curr_index = match self.dirlist.index {
+        let curr_index = match self.dirlist.get_index() {
             Some(i) => i,
             None => {
                 let style = Style::default().bg(Color::Red).fg(Color::White);
@@ -41,7 +53,16 @@ impl<'a> Widget for TuiDirListDetailed<'a> {
         };
 
         let drawing_width = area.width as usize;
-        let skip_dist = self.dirlist.first_index_for_viewport(area.height as usize);
+        let skip_dist = self.dirlist.first_index_for_viewport();
+        let line_num_style = self.display_options.line_nums();
+        // Length (In chars) of the last entry's index on current page.
+        // Using this to align all elements
+        let max_index_length = (skip_dist
+            + min(self.dirlist.len() - skip_dist, area.height as usize))
+        .to_string()
+        .len();
+
+        let space_fill = " ".repeat(drawing_width);
 
         // draw every entry
         self.dirlist
@@ -50,26 +71,44 @@ impl<'a> Widget for TuiDirListDetailed<'a> {
             .enumerate()
             .take(area.height as usize)
             .for_each(|(i, entry)| {
-                let style = style::entry_style(entry);
-                print_entry(buf, entry, style, (x + 1, y + i as u16), drawing_width - 1);
+                let ix = skip_dist + i;
+
+                let style = if !self.focused {
+                    style::entry_style(entry)
+                } else if ix == curr_index {
+                    style::entry_style(entry).add_modifier(Modifier::REVERSED)
+                } else {
+                    style::entry_style(entry)
+                };
+
+                buf.set_string(x, y + i as u16, space_fill.as_str(), style);
+
+                let mut prefix = if entry.is_selected() {
+                    " ".to_string()
+                } else {
+                    "".to_string()
+                };
+                let line_number_prefix = match line_num_style {
+                    LineNumberStyle::None => "".to_string(),
+                    _ if ix == curr_index => format!("{:<1$} ", curr_index + 1, max_index_length),
+                    LineNumberStyle::Absolute => format!("{:1$} ", ix + 1, max_index_length),
+                    LineNumberStyle::Relative => format!(
+                        "{:1$} ",
+                        (curr_index as i16 - ix as i16).abs(),
+                        max_index_length
+                    ),
+                };
+                prefix.push_str(&line_number_prefix);
+
+                print_entry(
+                    buf,
+                    entry,
+                    style,
+                    (x + 1, y + i as u16),
+                    drawing_width - 1,
+                    &prefix,
+                );
             });
-
-        // draw selected entry in a different style
-        let screen_index = curr_index % area.height as usize;
-
-        let entry = self.dirlist.curr_entry_ref().unwrap();
-        let style = style::entry_style(entry).add_modifier(Modifier::REVERSED);
-
-        let space_fill = " ".repeat(drawing_width);
-        buf.set_string(x, y + screen_index as u16, space_fill.as_str(), style);
-
-        print_entry(
-            buf,
-            entry,
-            style,
-            (x + 1, y + screen_index as u16),
-            drawing_width - 1,
-        );
     }
 }
 
@@ -79,6 +118,7 @@ fn print_entry(
     style: Style,
     (x, y): (u16, u16),
     drawing_width: usize,
+    prefix: &str,
 ) {
     let size_string = match entry.metadata.file_type() {
         FileType::Directory => entry
@@ -95,16 +135,23 @@ fn print_entry(
     let left_label_original = entry.label();
     let right_label_original = format!(" {}{} ", symlink_string, size_string);
 
+    // draw prefix first
+    let prefix_width = prefix.width();
+    buf.set_stringn(x, y, prefix, prefix_width, Style::default());
+    let x = x + prefix_width as u16;
+
+    // factor left_label and right_label
+    let drawing_width = drawing_width - prefix_width as usize;
     let (left_label, right_label) = factor_labels_for_entry(
         left_label_original,
         right_label_original.as_str(),
         drawing_width,
     );
 
-    let right_width = right_label.width();
+    // Draw labels
     buf.set_stringn(x, y, left_label, drawing_width, style);
     buf.set_stringn(
-        x + drawing_width as u16 - right_width as u16,
+        x + drawing_width as u16 - right_label.width() as u16,
         y,
         right_label,
         drawing_width,
@@ -127,22 +174,20 @@ fn factor_labels_for_entry<'a>(
         ("".to_string(), "")
     } else if width_remainder >= 0 {
         (left_label_original.to_string(), right_label_original)
+    } else if left_width_remainder < MIN_LEFT_LABEL_WIDTH {
+        (
+            if left_label_original.width() as i32 <= left_width_remainder {
+                trim_file_label(left_label_original, drawing_width)
+            } else {
+                left_label_original.to_string()
+            },
+            "",
+        )
     } else {
-        if left_width_remainder < MIN_LEFT_LABEL_WIDTH {
-            (
-                if left_label_original.width() as i32 <= left_width_remainder {
-                    trim_file_label(left_label_original, drawing_width)
-                } else {
-                    left_label_original.to_string()
-                },
-                "",
-            )
-        } else {
-            (
-                trim_file_label(left_label_original, left_width_remainder as usize),
-                right_label_original,
-            )
-        }
+        (
+            trim_file_label(left_label_original, left_width_remainder as usize),
+            right_label_original,
+        )
     }
 }
 
@@ -161,17 +206,19 @@ pub fn trim_file_label(name: &str, drawing_width: usize) -> String {
         truncated
     } else {
         let ext_width = extension.width();
-        if ext_width > drawing_width {
-            // file ext does not fit
-            let stem_width = drawing_width;
-            let truncated_stem = stem.trunc(stem_width - 3);
-            format!("{}{}.{}", truncated_stem, ELLIPSIS, ELLIPSIS)
-        } else if ext_width == drawing_width {
-            extension.replacen('.', ELLIPSIS, 1)
-        } else {
-            let stem_width = drawing_width - ext_width;
-            let truncated_stem = stem.trunc(stem_width - 1);
-            format!("{}{}{}", truncated_stem, ELLIPSIS, extension)
+        match ext_width.cmp(&drawing_width) {
+            Ordering::Less => {
+                let stem_width = drawing_width - ext_width;
+                let truncated_stem = stem.trunc(stem_width - 1);
+                format!("{}{}{}", truncated_stem, ELLIPSIS, extension)
+            }
+            Ordering::Equal => extension.replacen('.', ELLIPSIS, 1),
+            Ordering::Greater => {
+                // file ext does not fit
+                let stem_width = drawing_width;
+                let truncated_stem = stem.trunc(stem_width - 3);
+                format!("{}{}.{}", truncated_stem, ELLIPSIS, ELLIPSIS)
+            }
         }
     }
 }
