@@ -5,10 +5,10 @@ use tui::text::Span;
 use tui::widgets::{Clear, Paragraph, Wrap};
 
 use crate::context::AppContext;
+use crate::event::process_event;
 use crate::event::AppEvent;
 use crate::ui::views::TuiView;
-use crate::ui::TuiBackend;
-use crate::util::input;
+use crate::ui::AppBackend;
 
 pub struct TuiPrompt<'a> {
     prompt: &'a str,
@@ -19,7 +19,7 @@ impl<'a> TuiPrompt<'a> {
         Self { prompt }
     }
 
-    pub fn get_key(&mut self, backend: &mut TuiBackend, context: &mut AppContext) -> Key {
+    pub fn get_key(&mut self, backend: &mut AppBackend, context: &mut AppContext) -> Key {
         let terminal = backend.terminal_mut();
 
         context.flush_event();
@@ -62,7 +62,7 @@ impl<'a> TuiPrompt<'a> {
                     AppEvent::Termion(_) => {
                         context.flush_event();
                     }
-                    event => input::process_noninteractive(event, context),
+                    event => process_event::process_noninteractive(event, context),
                 };
             }
         }
