@@ -1,14 +1,11 @@
 #[cfg(feature = "devicons")]
 pub mod devicons;
 
-pub mod display;
 pub mod format;
-pub mod input;
 pub mod keyparse;
 pub mod name_resolution;
+pub mod process;
 pub mod search;
-pub mod select;
-pub mod sort;
 pub mod string;
 pub mod style;
 pub mod to_string;
